from gents.hfcollection import HFCollection
from gents.timeseries import TSCollection
import gents


def run_config(args):
    gents.utils.enable_logging(verbose=args.verbose)

    client = None
    if not args.serial:
        from dask.distributed import LocalCluster, Client
        cluster = LocalCluster(n_workers=args.numcores, threads_per_worker=1, memory_limit=f"{args.memorylimit}GB", processes=True)
        client = cluster.get_client()
    
    hf_collection = HFCollection(args.hf_head_dir, dask_client=client)
    hf_collection = hf_collection.include(["*/atm/*", "*/ice/*", "*/lnd/*", "*/glc/*", "*/rof/*"]).slice_groups(slice_size_years=args.slice)
    hf_collection.pull_metadata()
    
    ts_collection = TSCollection(hf_collection, args.outputdir, dask_client=client).apply_path_swap("/hist/", "/proc/tseries/").append_timestep_dirs()
    if args.overwrite:
        ts_collection = ts_collection.apply_overwrite("*")

    if not args.dryrun:
        ts_collection.execute()

    if not args.serial:
        client.shutdown()
        cluster.close()
    
    print("GenTS done!")


import argparse
import sys
from gents.utils import get_version

def parse_arguments():
    parser = argparse.ArgumentParser(
        description="GenTS "
    )
    parser.add_argument(
        "hf_head_dir",
        type=str,
        help="Path to head directory for history files."
    )
    parser.add_argument(
        "-o", "--outputdir",
        type=str,
        help="Path to the output time-series directory structure and files to. (Default is hf_head_dir)"
    )
    parser.add_argument(
        "-v", "--verbose",
        action="store_true",
        help="Enable verbose output."
    )
    parser.add_argument(
        "-V", "--version",
        action="version",
        version=f"%(prog)s {get_version()}"
    )
    parser.add_argument(
        "-d", "--dryrun",
        action="store_true",
        help="Reads and interprets history file metadata but does not generate time series files."
    )
    parser.add_argument(
        "-w", "--overwrite",
        action="store_true",
        help="Overwrite existing time series files if they exist."
    )
    parser.add_argument(
        "-sl", "--slice",
        type=int,
        default=10,
        help="Maximum length of individual time series files in years. (Default 10)"
    )
    parser.add_argument(
        "-s", "--serial",
        action="store_true",
        help="Disables Dask parallelism."
    )
    parser.add_argument(
        "-nc", "--numcores",
        type=int,
        default=8,
        help="Numer of cores to use if running in parallel. (Default 8)"
    )
    parser.add_argument(
        "-ml", "--memorylimit",
        type=int,
        default=2,
        help="Parallel per-core memory limit in GB. (Default 2)"
    )
    return parser.parse_args()


def main():
    args = parse_arguments()

    if args.outputdir is None:
        args.outputdir = args.hf_head_dir
    
    if args.verbose:
        print(f"  Input (HF) directory path    : {args.hf_head_dir}")
        print(f"  Output (TS) directory path   : {args.outputdir}")
        print(f"  Model Configuration          : CESM3")
        print(f"  Overwrite TS Files           : {args.overwrite}")
        print(f"  Slice size                   : {args.slice}")
        print(f"  Dry run                      : {args.dryrun}")
        print(f"  Disable Dask                 : {args.serial}")
        if not args.serial:
            print(f"  # Cores (Dask workers)       : {args.numcores}")
            print(f"  Per-core Memory              : {args.memorylimit}")
    
    from gents.configs.gents_cesm3 import run_config

    run_config(args)